/* cursor.c */

/* VIS-5D version 3.0 */

/*
vis5d program for visualizing five dimensional gridded data sets
Copyright (C) 1990, 1991, 1992  Bill Hibbard, Brian Paul, Dave Santek,
and Andre Battaiola.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include <stdio.h>
#include <X11/Xlib.h>
#include "globals.h"
#include "graphics.h"




/*** cursor_event ********************************************************
   This function receives an input event and tries to use it to update
   the cursor's position.
   Input: event - a copy of the XEvent.
   Returned:  0 = event not used.
              1 = cursor moved.
	      2 = middle button pressed.
*************************************************************************/
int cursor_event( event )
XEvent event;
{
   static int start_y;
   static int pressed3 = 0;
   int now_x, now_y;      /* mouse position */
   float delta_y;         /* mouse movement vector */
   int result;           /* value returned */
   float v[4], p[4];
   float x,y,z;

   result = 0;

   if (event.type==ButtonPress && event.xbutton.button==Button3) {
      /* Right mouse button has been pressed. */
      pressed3 = 1;
      result = 1;
   }
   else if (event.type==ButtonPress && event.xbutton.button==Button2) {
      /* middle button pressed */
      result = 2;
   }
   else if (event.type==ButtonRelease && event.xbutton.button==Button3) {
      /* Right mouse button has been released. */
      pressed3 = 0;
      result = 1;
   }

   if ( (event.type==MotionNotify || event.type==ButtonPress) && pressed3 ) {
      /* Mouse has been moved while middle button held down. */
      now_x = event.xbutton.x;    /* save current mouse position. */
      now_y = event.xbutton.y;
      /* right button -> reposition cursor */
      /* convert 3-D cursor's position to screen coordinates */
      p[0] = CursorX;  p[1] = CursorY;  p[2] = CursorZ;
      mat_vecmul( p, CTM );
      p[0] /= AspectRatio;
      /* vector from pointer position to cursor position: */
      v[0] = (now_x*2.0 / Width - 1.0) - p[0];
      v[1] = (1.0 - now_y*2.0 / Height) - p[1];
      v[2] = 0.0;
      mat_vecmul( v, CTMinv );
      CursorX += v[0] - CTM[3][0];
      CursorY += v[1] - CTM[3][1];
      CursorZ += v[2] - CTM[3][2];
      /* make sure cursor is in the box. */
      if (CursorX>Xmax)  CursorX = Xmax;
      if (CursorX<Xmin)  CursorX = Xmin;
      if (CursorY>Ymax)  CursorY = Ymax;
      if (CursorY<Ymin)  CursorY = Ymin;
      if (CursorZ>Zmax)  CursorZ = Zmax;
      if (CursorZ<Zmin)  CursorZ = Zmin;

      /* convert position from graphics to grid and geo coords */
      x = (CursorX-Xmin) / (Xmax-Xmin);
      y = (Ymax-CursorY) / (Ymax-Ymin);
      z = (Zmax-CursorZ) / (Zmax-Zmin);
      CursorR = y * (Nr-1);
      CursorC = x * (Nc-1);
      CursorL = z * (Nl-1);

      result = 1;
   }

   return result;
}


